@@ -31,7 +31,6 @@ def parse_args(args):
         description="Initializes an S3QL file system")
 
     parser.add_cachedir()
-    parser.add_authfile()
     parser.add_debug()
     parser.add_quiet()
     parser.add_backend_options()
@@ -95,7 +94,7 @@ def main(args=None):
     atexit.register(plain_backend.close)
 
     log.info("Before using S3QL, make sure to read the user's guide, especially\n"
-             "the 'Important Rules to Avoid Loosing Data' section.")
+             "the 'Important Rules to Avoid Losing Data' section.")
 
     if isinstance(plain_backend, s3.Backend) and '.' in plain_backend.bucket_name:
         log.warning('S3 Buckets with names containing dots cannot be '
