@@ -1,10 +1,10 @@
 #!/usr/bin/env python3
 '''
-t1_dump.py - this file is part of S3QL (http://s3ql.googlecode.com)
+t1_dump.py - this file is part of S3QL.
 
-Copyright (c) Nikolaus Rath <Nikolaus@rath.org>
+Copyright © 2008 Nikolaus Rath <Nikolaus@rath.org>
 
-This program can be distributed under the terms of the GNU GPLv3.
+This work can be distributed under the terms of the GNU GPLv3.
 '''
 
 if __name__ == '__main__':
@@ -30,7 +30,7 @@ class DumpTests(unittest.TestCase):
         # Disable exclusive locking for all tests
         self.src.execute('PRAGMA locking_mode = NORMAL')
         self.dst.execute('PRAGMA locking_mode = NORMAL')
-        
+
         self.create_table(self.src)
         self.create_table(self.dst)
 
@@ -43,7 +43,7 @@ class DumpTests(unittest.TestCase):
 
     def test_transactions(self):
         self.fill_vals(self.src)
-        dumpspec = (('id', deltadump.INTEGER, 0),)        
+        dumpspec = (('id', deltadump.INTEGER, 0),)
         deltadump.dump_table(table='test', order='id', columns=dumpspec,
                              db=self.src, fh=self.fh)
         self.fh.seek(0)
@@ -52,7 +52,7 @@ class DumpTests(unittest.TestCase):
         deltadump.load_table(table='test', columns=dumpspec, db=self.dst,
                              fh=self.fh, trx_rows=10)
         self.compare_tables(self.src, self.dst)
-                
+
     def test_1_vals_1(self):
         self.fill_vals(self.src)
         dumpspec = (('id', deltadump.INTEGER, 0),)
@@ -138,27 +138,6 @@ class DumpTests(unittest.TestCase):
                              fh=self.fh)
         self.compare_tables(self.src, self.dst)
 
-    def test_4_time(self):
-        self.fill_vals(self.src)
-
-        t1 = 0.5 * time.time()
-        t2 = 2 * time.time()
-        for (id_,) in self.src.query('SELECT id FROM test'):
-            val = random.uniform(t1, t2)
-            self.src.execute('UPDATE test SET buf=? WHERE id=?', (val, id_))
-
-        dumpspec = (('id', deltadump.INTEGER),
-                    ('buf', deltadump.TIME))
-
-        deltadump.dump_table(table='test', order='id', columns=dumpspec,
-                             db=self.src, fh=self.fh)
-        self.fh.seek(0)
-        deltadump.load_table(table='test', columns=dumpspec, db=self.dst,
-                             fh=self.fh)
-
-        self.compare_tables(self.src, self.dst)
-
-
     def test_5_multi(self):
         self.fill_vals(self.src)
         dumpspec = (('id', deltadump.INTEGER, 0),)
@@ -201,7 +180,7 @@ class DumpTests(unittest.TestCase):
                     val = rfh.read(random.randint(0, 140))
                 else:
                     val = rfh.read(len_)
-    
+
                 db.execute('UPDATE test SET buf=? WHERE id=?', (val, id_))
 
     def fill_vals(self, db):
